/**
 * A Promise
 * @typedef {object} promise
 */

module.exports = class API {
    constructor (origin) {
        Object.assign(this, {
            routes: {
                get: [],
                post: [],
                put: [],
                patch: [],
                delete: []
            },
            middlewares: [],
            origin
        });
        for (let method of ['get', 'post', 'put', 'patch', 'delete']) {
            this[method] = (route, callback) => this.routes[method].push({
                route: parseRoute(route),
                callback
            });
        }
    }
    use (middleware) {
        this.middlewares.push(middleware);
    }
    /**
     * Delux store api middleware
     * @param {object} action - Delux action.
     * @returns {promise}
     */
    middleware (action) {
        if (action.api) {
            let method = Object.keys(action.api)[0];
            let settings = action.api[method];
            let url;
            let body;
            if (typeof settings == 'string') {
                url = settings;
            }
            else {
                url = settings.url;
                body = settings.body;
            }
            let fetched = fetch(this.origin + url, {method, body});
            fetched = fetched.then(res => {
                for (let route of this.routes[method]) {
                    if (url.search(route.regexp) > -1) {
                        return route.callback(res, action);
                    }
                }
            });
            for (let middleware of this.middlewares) {
                fetched = fetched.then(middleware);
            }
            return fetched;
        }
    }
};

/**
 * Transforms route string to route regexp
 * @param {string} route string
 * @returns {object} route regular expression
 */
function parseRoute (route) {
    return new RegExp(route.replace(/(\:[^/]+)|(?:\*)/g, '[^/]+'));
}
